﻿/**
* FrameScriptManager by Grant Skinner. November 13, 2007
* Visit www.gskinner.com/blog for documentation, updates and more free code.
*
* You may distribute this class freely, provided it is not modified in any way (including
* removing this header or changing the package path).
*
* Please contact info@gskinner.com prior to distributing modified versions of this class.
*/

package {

	import flash.display.MovieClip;
	import flash.utils.getQualifiedClassName;

	public class FrameScriptManager {
	// constants:
	// private properties:
		private var target:MovieClip;
		private var labels:Object;
		
	// public properties:
		
	// constructor:
		public function FrameScriptManager(target:MovieClip) {
			this.target = target;
			updateLabels();
		}
		
	// public getter/setters:
	// public methods:
		public function setFrameScript(frame:*,funct:Function):void {
			var frameNum:uint = getFrameNumber(frame);
			if (frameNum == 0) { return; }
			target.addFrameScript(frameNum-1,funct);
		}
		
		public function revertFrameScript(frame:*):void {
			var frameNum:uint = getFrameNumber(frame);
			if (frameNum == 0) { return; }
			var funct:Function = getOriginalFrameScript(frameNum);
			target.addFrameScript(frameNum-1,funct);
		}
		
		public function callOriginalFrameScript(frame:*):void {
			var funct:Function = getOriginalFrameScript(frame);
			if (funct != null) { funct(); }
		}
		
		public function getFrameNumber(frame:*):uint {
			var frameNum:uint = uint(frame);
			if (frameNum) { return frameNum; }
			var label:String = String(frame);
			if (label == null) { return 0; }
			frameNum = labels[label];
			return frameNum;
		}
		
		public function getOriginalFrameScript(frame:*):Function {
			var frameNum:uint = getFrameNumber(frame);
			if (frameNum == 0) { return null; }
			var functName:String = "frame"+frameNum;
			// frameN methods are internal, so they can't be accessed unless the target is in the same package as FrameScriptManager
			// they also can't be inspected with "in" or "hasOwnProperty" to determine if frame script exists, so we'll use a try...catch block instead.
			var funct:Function = null;
			try { funct = target[functName]; } catch (e:*) {
				// error accessing the frameN method, lets check if the two classes are in the same package, and throw an appropriate error:
				var pkg:String = getQualifiedClassName(this);
				var index:int = pkg.lastIndexOf("::");
				pkg = (index == -1) ? "" : pkg.substr(0,index);
				var tpkg:String = getQualifiedClassName(this);
				index = tpkg.lastIndexOf("::");
				tpkg = (index == -1) ? "" : tpkg.substr(0,index);
				if (tpkg == pkg) {
					throw(new ReferenceError("Property "+functName+" not found on target MovieClip."));
				} else {
					throw(new Error("getOriginalFrameScript only works with targets in the same package [target package="+tpkg+"]."));
				}
			}
			return funct;
		}
	
	// private methods:
		private function updateLabels():void {
			var lbls:Array = target.currentLabels;
			labels = {};
			for (var i:uint=0; i<lbls.length; i++) {
				labels[lbls[i].name] = lbls[i].frame;
			}
		}
	}
}